@@ -1,19 +1,58 @@
 package CHI;
+$CHI::VERSION = '0.61';
 use 5.006;
 use Carp;
 use CHI::Stats;
+use String::RewritePrefix;
+use Module::Runtime qw(require_module);
+use Moo::Role ();
 use strict;
 use warnings;
 
-our $VERSION = '0.36';
-
-my ( %final_class_seen, %stats );
+my ( %final_class_seen, %memoized_cache_objects, %stats );
+my %valid_config_keys =
+  map { ( $_, 1 ) } qw(defaults memoize_cache_objects namespace storage);
 
 sub logger {
     warn
       "CHI now uses Log::Any for logging - see Log::Any documentation for details";
 }
 
+sub config {
+    my $class = shift;
+    $class->_set_config(@_) if @_;
+    return $class->_get_config();
+}
+
+sub _set_config {
+    my ( $class, $config ) = @_;
+    if ( my @bad_keys = grep { !$valid_config_keys{$_} } keys(%$config) ) {
+        croak "unknown keys in config hash: " . join( ", ", @bad_keys );
+    }
+
+    # set class specific configuration
+    no strict 'refs';
+    no warnings 'redefine';
+    *{"$class\::_get_config"} = sub { $config };
+}
+
+BEGIN { __PACKAGE__->config( {} ) }
+
+sub memoized_cache_objects {
+    my ($class) = @_;
+
+    # Each CHI root class gets its hash of memoized objects
+    #
+    $memoized_cache_objects{$class} ||= {};
+    return $memoized_cache_objects{$class};
+}
+
+sub clear_memoized_cache_objects {
+    my ($class) = @_;
+
+    $memoized_cache_objects{$class} = {};
+}
+
 sub stats {
     my ($class) = @_;
 
@@ -26,11 +65,48 @@ sub stats {
 sub new {
     my ( $chi_root_class, %params ) = @_;
 
+    my $config = $chi_root_class->config;
+
+    # Cache object memoization: See if cache object with these parameters
+    # has already been created, and return it if so. Only for parameters
+    # with 0 or 1 keys.
+    #
+    my ( $cache_object_key, $cache_objects );
+    if ( $config->{memoize_cache_objects} && keys(%params) <= 1 ) {
+        $cache_object_key = join chr(28), %params;
+        $cache_objects = $chi_root_class->memoized_cache_objects;
+        if ( my $cache_object = $cache_objects->{$cache_object_key} ) {
+            return $cache_object;
+        }
+    }
+
+    # Gather defaults
+    #
+    my $core_defaults = $config->{defaults} || {};
+    my $namespace_defaults =
+      $config->{namespace}->{ $params{namespace} || 'Default' } || {};
+    my $storage =
+         $params{storage}
+      || $namespace_defaults->{storage}
+      || $core_defaults->{storage};
+    my $storage_defaults = {};
+    if ( defined($storage) ) {
+        $storage_defaults = $config->{storage}->{$storage}
+          or croak "no config for storage type '$storage'";
+    }
+
+    # Combine passed params with defaults
+    #
+    %params =
+      ( %$core_defaults, %$storage_defaults, %$namespace_defaults, %params );
+
     # Get driver class from driver or driver_class parameters
     #
     my $driver_class;
     if ( my $driver = delete( $params{driver} ) ) {
-        $driver_class = "CHI::Driver::$driver";
+        ($driver_class) =
+          String::RewritePrefix->rewrite( { '' => 'CHI::Driver::', '+' => '' },
+            $driver );
     }
     else {
         $driver_class = delete( $params{driver_class} );
@@ -41,43 +117,50 @@ sub new {
     # Load driver class if it hasn't been loaded or defined in-line already
     #
     unless ( $driver_class->can('fetch') ) {
-        Class::MOP::load_class($driver_class);
+        require_module($driver_class);
     }
 
     # Select roles depending on presence of certain arguments. Everyone gets
-    # the Universal role.
+    # the Universal role. Accept both 'roles' and 'traits' for backwards
+    # compatibility. Add CHI::Driver::Role:: unless prefixed with '+'.
     #
-    my @roles = ('CHI::Driver::Role::Universal');
-    if ( exists( $params{roles} ) ) {
-        push( @roles, @{ delete( $params{roles} ) } );
+    my @roles = ('Universal');
+    foreach my $param_name (qw(roles traits)) {
+        if ( exists( $params{$param_name} ) ) {
+            push( @roles, @{ delete( $params{$param_name} ) } );
+        }
     }
     if ( exists( $params{max_size} ) || exists( $params{is_size_aware} ) ) {
-        push( @roles, 'CHI::Driver::Role::IsSizeAware' );
+        push( @roles, 'IsSizeAware' );
     }
     if ( exists( $params{l1_cache} ) || exists( $params{mirror_cache} ) ) {
-        push( @roles, 'CHI::Driver::Role::HasSubcaches' );
+        push( @roles, 'HasSubcaches' );
     }
     if ( $params{is_subcache} ) {
-        push( @roles, 'CHI::Driver::Role::IsSubcache' );
+        push( @roles, 'IsSubcache' );
     }
+    @roles = String::RewritePrefix->rewrite(
+        { '' => 'CHI::Driver::Role::', '+' => '' }, @roles );
 
     # Select a final class based on the driver class and roles, creating it
     # if necessary - adapted from MooseX::Traits
     #
-    my $meta = Moose::Meta::Class->create_anon_class(
-        superclasses => [$driver_class],
-        roles        => \@roles,
-        cache        => 1
-    );
-    my $final_class = $meta->name;
-    $meta->add_method( 'meta' => sub { $meta } )
-      if !$final_class_seen{$final_class}++;
+    my $final_class =
+      Moo::Role->create_class_with_roles( $driver_class, @roles );
 
-    return $final_class->new(
+    my $cache_object = $final_class->new(
         chi_root_class => $chi_root_class,
         driver_class   => $driver_class,
         %params
     );
+
+    # Memoize if appropriate
+    #
+    if ($cache_object_key) {
+        $cache_objects->{$cache_object_key} = $cache_object;
+    }
+
+    return $cache_object;
 }
 
 1;
@@ -88,7 +171,11 @@ __END__
 
 =head1 NAME
 
-CHI -- Unified cache handling interface
+CHI - Unified cache handling interface
+
+=head1 VERSION
+
+version 0.61
 
 =head1 SYNOPSIS
 
@@ -97,6 +184,7 @@ CHI -- Unified cache handling interface
     # Choose a standard driver
     #
     my $cache = CHI->new( driver => 'Memory', global => 1 );
+    my $cache = CHI->new( driver => 'RawMemory', global => 1 );
     my $cache = CHI->new( driver => 'File',
         root_dir => '/path/to/root'
     );
@@ -104,7 +192,7 @@ CHI -- Unified cache handling interface
         root_dir   => '/path/to/root',
         cache_size => '1k'
     );
-    my $cache = CHI->new( driver  => 'Memcached',
+    my $cache = CHI->new( driver  => 'Memcached::libmemcached',
         servers => [ "10.0.0.15:11211", "10.0.0.15:11212" ],
         l1_cache => { driver => 'FastMmap', root_dir => '/path/to/root' }
     );
@@ -117,15 +205,18 @@ CHI -- Unified cache handling interface
 
     # Create your own driver
     # 
-    my $cache = CHI->new( driver_class => 'My::Special::Driver' );
+    my $cache = CHI->new( driver => '+My::Special::Driver', ... );
 
-    # Basic cache operations
+    # Cache operations
     #
     my $customer = $cache->get($name);
     if ( !defined $customer ) {
         $customer = get_customer_from_db($name);
         $cache->set( $name, $customer, "10 minutes" );
     }
+    my $customer2 = $cache->compute($name2, "10 minutes", sub {
+        get_customer_from_db($name2)
+    });
     $cache->remove($name);
 
 =head1 DESCRIPTION
@@ -177,30 +268,58 @@ Optional logging and statistics collection of cache activity
 
 =head1 CONSTRUCTOR
 
-To create a new cache handle, call CHI-E<gt>new. It takes the following common
-options. All are optional, except that either I<driver> or I<driver_class> must
-be passed.
+To create a new cache object, call C<CHI-E<gt>new>. It takes the common
+options listed below. I<driver> is required; all others are optional.
+
+Some drivers will take additional constructor options. For example, the File
+driver takes C<root_dir> and C<depth> options.
+
+You can configure default options for each new cache object created - see
+L</SUBCLASSING AND CONFIGURING CHI>.
+
+Note that C<CHI-E<gt>new> returns an instance of a subclass of
+L<CHI::Driver|CHI::Driver>, not C<CHI>.
 
 =over
 
+=item compress_threshold [INT]
+
+A value in bytes. Automatically compress values larger than this before
+storing.  Requires L<Compress::Zlib|Compress::Zlib> to be installed. Defaults
+to undef, meaning no automatic compression. Inspired by the parameter of the
+same name in L<Cache::Memcached>.
+
+    # Compress values larger than 1MB
+    compress_threshold => 1024*1024
+
 =item driver [STRING]
 
-The name of a standard driver to drive the cache, for example "Memory" or
-"File".  CHI will prefix the string with "CHI::Driver::".
+Required. The name of a cache driver, for example "Memory" or "File".  CHI will
+prefix the string with "CHI::Driver::", unless it begins with '+'. e.g.
 
-=item driver_class [STRING]
+    driver => 'File';                   # uses CHI::Driver::File
+    driver => '+My::CHI::Driver::File'  # uses My::CHI::Driver::File
 
-The exact CHI::Driver subclass to drive the cache, for example
-"My::Memory::Driver".
-
-=item expires_in [DURATION]
-
-=item expires_at [INT]
-
-=item expires_variance [FLOAT]
+=item expires_in [DURATION], expires_at [INT], expires_variance [FLOAT]
 
 Provide default values for the corresponding L</set> options.
 
+=item expires_on_backend [NUM]
+
+If set to 0 (the default), CHI alone is aware of the expiration time and does
+not pass it along to the backend driver. This allows you to use L</get_object>
+to retrieve expired items.
+
+If set to 1, pass expiration times to backend driver if the driver supports it
+-- for example, L<CHI::Driver::Memcached|Memcached> and
+L<CHI::Driver::CacheCache|CacheCache>. This may allow the driver to better
+manage its space and evict items. Note that only simple expiration time will be
+passed along, e.g. not L</expires_variance>.
+
+If set to a number greater than 1 (e.g. 1.25), the time until expiration will
+be multiplied by that number before being passed to the backend driver. This
+gives you a customizable window of opportunity to retrieve expired items.
+
 =item key_digester [STRING|HASHREF|OBJECT]
 
 Digest algorithm to use on keys longer than L</max_key_length> - e.g. "MD5",
@@ -212,11 +331,18 @@ options.
 
 Default is "MD5".
 
+=item key_serializer [STRING|HASHREF|OBJECT]
+
+An object to use for serializing keys that are references. See L</serializer>
+above for the different ways this can be passed in. The default is to use the
+JSON backend in canonical mode (sorted hash keys).
+
 =item label [STRING]
 
 A label for the cache as a whole, independent of namespace - e.g.
-"web-file-cache". Used when referring to the cache in logs, statistics, and
-error messages. By default, set to L</short_driver_name>.
+"web-file-cache". Used when referring to the cache in logs,
+L<statistics|CHI::Stats>, and error messages. By default, set to
+L</short_driver_name>.
 
 =item l1_cache [HASHREF]
 
@@ -264,39 +390,6 @@ will set the namespace to the current component path.
 
 Defaults to 'Default' if not specified.
 
-=item serializer [STRING|HASHREF|OBJECT]
-
-An object to use for serializing data before storing it in the cache, and
-deserializing data after retrieving it from the cache.
-
-If this is a string, a L<Data::Serializer|Data::Serializer> object will be
-created, with the string passed as the 'serializer' option and raw=1. Common
-options include 'Storable', 'Data::Dumper', and 'YAML'. If this is a hashref,
-L<Data::Serializer|Data::Serializer-E<gt>new> will be called with the hash. You
-will need to ensure Data::Serializer is installed to use these options.
-
-Otherwise, this must be a L<Data::Serializer|Data::Serializer> object or
-another object that implements I<serialize()> and I<deserialize()>.
-
-e.g.
-
-    # Serialize using raw Data::Dumper
-    my $cache = CHI->new(serializer => 'Data::Dumper');
-
-    # Serialize using Data::Dumper, compressed and (per Data::Serializer defaults) hex-encoded
-    my $cache = CHI->new(serializer => { serializer => 'Data::Dumper', compress => 1 });
-
-    # Serialize using custom object
-    my $cache = CHI->new(serializer => My::Custom::Serializer->new())
-
-The default is to use raw Storable.
-
-=item key_serializer [STRING|HASHREF|OBJECT]
-
-An object to use for serializing keys that are references. See L</serializer>
-above for the different ways this can be passed in. The default is to use JSON
-in canonical mode (sorted hash keys).
-
 =item on_get_error [STRING|CODEREF]
 
 =item on_set_error [STRING|CODEREF]
@@ -330,10 +423,43 @@ message, the key, and the original raw error message
 
 =back
 
-=back    
+=item serializer [STRING|HASHREF|OBJECT]
 
-Some drivers will take additional constructor options. For example, the File
-driver takes C<root_dir> and C<depth> options.
+An object to use for serializing data before storing it in the cache, and
+deserializing data after retrieving it from the cache. Only references will be
+serialized; plain scalars will be placed in the cache as-is.
+
+If this is a string, a L<Data::Serializer|Data::Serializer> object will be
+created, with the string passed as the 'serializer' option and raw=1. Common
+options include 'Storable', 'Data::Dumper', and 'YAML'. If this is a hashref,
+L<Data::Serializer-E<gt>new|Data::Serializer> will be called with the hash. You
+will need to ensure Data::Serializer is installed to use these options.
+
+Otherwise, this must be a L<Data::Serializer|Data::Serializer> object or
+another object that implements I<serialize()> and I<deserialize()>.
+
+e.g.
+
+    # Serialize using raw Data::Dumper
+    my $cache = CHI->new(serializer => 'Data::Dumper');
+
+    # Serialize using Data::Dumper, compressed and (per Data::Serializer defaults) hex-encoded
+    my $cache = CHI->new(serializer => { serializer => 'Data::Dumper', compress => 1 });
+
+    # Serialize using custom object
+    my $cache = CHI->new(serializer => My::Custom::Serializer->new())
+
+The default is to use raw Storable.
+
+=item traits [LISTREF]
+
+List of one or more roles to apply to the C<CHI::Driver> class that is
+constructed. The roles will automatically be prefixed with
+C<CHI::Driver::Role::> unless preceded with a '+'. e.g.
+
+    traits => ['StoresAccessedAt', '+My::CHI::Driver::Role']
+
+=back
 
 =head1 INSTANCE METHODS
 
@@ -357,20 +483,26 @@ I<$key> may be followed by one or more name/value parameters:
 =item expire_if [CODEREF]
 
 If I<$key> exists and has not expired, call code reference with the
-L<CHI::CacheObject|CHI::CacheObject> as a single parameter. If code returns a
-true value, C<get> returns undef as if the item were expired. For example, to
-treat the cache as expired if I<$file> has changed since the value was
-computed:
+L<CHI::CacheObject|CHI::CacheObject> and L<CHI::Driver|CHI::Driver> as the
+parameters. If code returns a true value, C<get> returns undef as if the item
+were expired. For example, to treat the cache as expired if I<$file> has
+changed since the value was computed:
 
     $cache->get('foo', expire_if => sub { $_[0]->created_at < (stat($file))[9] });
 
 =item busy_lock [DURATION]
 
-If the value has expired, set its expiration time to the current time plus the
-specified L<duration|/DURATION EXPRESSIONS> before returning undef.  This is
-used to prevent multiple processes from recomputing the same expensive value
-simultaneously. The problem with this technique is that it doubles the number
-of writes performed - see L</expires_variance> for another technique.
+If the value has expired, the get will still return undef, but the expiration
+time of the cache entry will be set to the current time plus the specified
+L<duration|/DURATION EXPRESSIONS>.  This is used to prevent multiple processes
+from recomputing the same expensive value simultaneously. The problem with this
+technique is that it doubles the number of writes performed - see
+L</expires_variance> for another technique.
+
+=item obj_ref [SCALARREF]
+
+If the item exists in cache (even if expired), place the
+L<CHI::CacheObject|CHI::CacheObject> object in the provided SCALARREF.
 
 =back
 
@@ -389,7 +521,8 @@ these options can be provided with defaults in the cache constructor.
 
 =item expires_in [DURATION]
 
-Amount of time (in seconds) until this data expires.
+Amount of time from now until this data expires. I<DURATION> may be an integer
+number of seconds or a L<duration expression|/DURATION EXPRESSIONS>.
 
 =item expires_at [INT]
 
@@ -402,9 +535,9 @@ earlier than the stated expiration time to help prevent cache miss stampedes.
 
 Value is between 0.0 and 1.0, with 0.0 meaning that items expire exactly when
 specified (feature is disabled), and 1.0 meaning that items might expire
-anytime from now til the stated expiration time. The default is 0.0. A setting
-of 0.10 to 0.25 would introduce a small amount of variation without interfering
-too much with intended expiration times.
+anytime from now until the stated expiration time. The default is 0.0. A
+setting of 0.10 to 0.25 would introduce a small amount of variation without
+interfering too much with intended expiration times.
 
 The probability of expiration increases as a function of how far along we are
 in the potential expiration window, with the probability being near 0 at the
@@ -436,18 +569,49 @@ as soon as it is determined to be expired. But it's something to be aware of.
 
 =back
 
-=item compute( $key, $code, $set_options )
+=item compute( $key, $options, $code )
 
 Combines the C<get> and C<set> operations in a single call. Attempts to get
 I<$key>; if successful, returns the value. Otherwise, calls I<$code> and uses
-the return value as the new value for I<$key>, which is then returned.
-I<$set_options> is a scalar or hash reference, used as the third argument to
-set.
+the return value as the new value for I<$key>, which is then returned. Caller
+context (scalar or list) is respected.
+
+I<$options> can be undef, a scalar, or a hash reference. If it is undef, it has
+no effect. If it is a scalar, it is treated as the C<expires_in> duration and
+passed as the third argument to C<set>. If it is a hash reference, it may
+contain name/value pairs for both C<get> and C<set>.  e.g.
+
+    # No expiration
+    my $value = $cache->compute($key, undef, sub {
+        # compute and return value for $key here
+    });
+
+    # Expire in 5 minutes
+    my $value = $cache->compute($key, '5min', sub {
+        # compute and return value for $key here
+    });
+
+    # Expire in 5 minutes or when a particular condition occurs
+    my $value = $cache->compute($key,
+        { expires_in => '5min', expire_if => sub { ... } },
+        sub {
+           # compute and return value for $key here
+    });
+
+    # List context
+    my @value = $cache->compute($key, '5min', sub {
+        ...
+        return @some_list;
+    });
 
 This method will eventually support the ability to recompute a value in the
 background just before it actually expires, so that users are not impacted by
 recompute time.
 
+Note: Prior to version 0.40, the last two arguments were in reverse order; both
+will be accepted for backward compatibility. We think the coderef looks better
+at the end.
+
 =back
 
 =head2 Removing and expiring
@@ -495,6 +659,54 @@ be returned even if the entry has expired, as long as it has not been removed.
 
 =back
 
+=head2 Atomic operations (ALPHA)
+
+These methods combine both reading and writing of a cache entry in a single
+operation. The names and behaviors were adapted from
+L<memcached|http://memcached.org/>.
+
+Some drivers (e.g.
+L<CHI::Driver::Memcached::libmemcached|Memcached::libmemcached>,
+L<CHI::Driver::DBI|DBI>) may implement these as truly atomic operations, and
+will be documented thusly.  The default implementations are not atomic: the get
+and set occur discretely and another process could potentially modify the cache
+in between them.
+
+These operations are labeled ALPHA because we haven't yet figured out how they
+integrate with other CHI features, in particular L</SUBCACHES>. APIs and
+behavior may change.
+
+=over
+
+=item add( $key, $data, [$expires_in | "now" | "never" | options] )
+
+Do a L<set>, but only if I<$key> is not L<valid|is_valid> in the cache.
+
+=item replace( $key, $data, [$expires_in | "now" | "never" | options] )
+
+Do a L<set>, but only if I<$key> is L<valid|is_valid> in the cache.
+
+=item append( $key, $new_data)
+
+Append I<$new_data> to whatever value is currently associated with I<$key>. Has
+no effect if I<$key> does not exist in the cache.
+
+Returns true if I<$key> was in the cache, false otherwise.
+
+This is intended for simple string values only. For efficiency's sake, CHI
+won't attempt to check for, or handle, the case where data is
+L<serialized|serializer> or L<compressed|compress_threshold>; the new data will
+simply be appended, and an error will most probably occur when you try to
+retrieve the value.
+
+Does not modify expiration or other metadata. If I<$key> exists but is expired,
+it will remain expired.
+
+If you use a driver with the non-atomic (default) implementation, some appends
+may be lost due to race conditions.
+
+=back
+
 =head2 Namespace operations
 
 =over
@@ -570,15 +782,20 @@ cache.
 
 =over
 
+=item chi_root_class( )
+
+Returns the name of the root class under which this object was created, e.g.
+C<CHI> or C<My::CHI>. See L</SUBCLASSING AND CONFIGURING CHI>.
+
 =item driver_class( )
 
 Returns the full name of the driver class. e.g.
 
     CHI->new(driver=>'File')->driver_class
        => CHI::Driver::File
-    CHI->new(driver_class=>'CHI::Driver::File')->driver_class
+    CHI->new(driver=>'+CHI::Driver::File')->driver_class
        => CHI::Driver::File
-    CHI->new(driver_class=>'My::Driver::File')->driver_class
+    CHI->new(driver=>'+My::Driver::File')->driver_class
        => My::Driver::File
 
 You should use this rather than C<ref()>. Due to some subclassing tricks CHI
@@ -605,12 +822,12 @@ e.g.
     label
     on_get_error
     on_set_error
-    
+
 =item Standard read-only accessors
 
     namespace
     serializer
-    
+
 =back
 
 =head2 Deprecated methods
@@ -707,7 +924,7 @@ places an in-process Memory cache in front of a Memcached cache:
     my $cache = CHI->new(
         driver   => 'Memcached',
         servers  => [ "10.0.0.15:11211", "10.0.0.15:11212" ],
-        l1_cache => { driver => 'Memory' }
+        l1_cache => { driver => 'Memory', global => 1, max_size => 1024*1024 }
     );
 
 On a C<get>, the L1 cache is checked first - if a valid value exists, it is
@@ -760,7 +977,7 @@ contain a hash of options to create the subcache with.
 The cache containing the subcache is called the I<parent cache>.
 
 The following options are automatically inherited by the subcache from the
-parent cache, and may not be overriden:
+parent cache, and may not be overridden:
 
     expires_at
     expires_in
@@ -771,10 +988,10 @@ parent cache, and may not be overriden:
 object|CHI::CacheObject> and store it in both caches. The cache object contains
 expiration information and is dependent on the serializer.  At some point we
 could conceivably add code that will use a single object or separate objects as
-necessary, and thus allow the above to be overriden.)
+necessary, and thus allow the above to be overridden.)
 
 The following options are automatically inherited by the subcache from the
-parent cache, but may be overriden:
+parent cache, but may be overridden:
 
     namespace
     on_get_error
@@ -823,7 +1040,7 @@ have its own subcaches, and so on. e.g.
         l1_cache => {
             driver     => 'File',
             root_dir   => '/path/to/root',
-            l1_cache   => { driver => 'Memory' }
+            l1_cache   => { driver => 'RawMemory', global => 1 }
         }
     );
 
@@ -916,6 +1133,186 @@ Also, for drivers that cannot atomically read and update a value - for example,
 L<CHI::Driver::File|File> - there is a race condition in the updating of size
 that can cause the size to grow inaccurate over time.
 
+=head1 SUBCLASSING AND CONFIGURING CHI
+
+You can subclass CHI for your own application and configure it in a variety of
+ways, e.g. predefining storage types and defaults for new cache objects. Your
+configuration will be independent of the main CHI class and other CHI
+subclasses.
+
+Start with a trivial subclass:
+
+    package My::CHI;
+
+    use base qw(CHI);
+    1;
+
+Then, just use your subclass in place of CHI:
+
+    my $cache = My::CHI->new( ... );
+
+    print $cache->chi_root_class;
+       ==> 'My::CHI'
+
+This obviously doesn't change any behavior by itself. Here's an example with
+actual config:
+
+    package My::CHI;
+
+    use base qw(CHI);
+
+    __PACKAGE__->config({
+        storage   => {
+            local_file => { driver => 'File', root_dir => '/my/root' },
+            memcached  => {
+                driver  => 'Memcached::libmemcached',
+                servers => [ '10.0.0.15:11211', '10.0.0.15:11212' ]
+            },
+        },
+        namespace => {
+            'Foo' => { storage => 'local_file' },
+            'Bar' => { storage => 'local_file', depth => 3 },
+            'Baz' => { storage => 'memcached' },
+        }
+        defaults  => { storage => 'local_file' },
+        memoize_cache_objects => 1,
+    });
+
+    1;
+
+Each of these config keys is explained in the next section.
+
+=head2 Configuration keys
+
+=over
+
+=item storage
+
+A map of names to parameter hashrefs. This provides a way to encapsulate common
+sets of parameters that might be used in many caches. e.g. if you define
+
+    storage => {
+        local_file => { driver => 'File', root_dir => '/my/root' },
+        ...
+    }
+
+then
+
+    my $cache = My::CHI->new
+       (namespace => 'Foo', storage => 'local_file');
+
+is equivalent to
+
+    my $cache = My::CHI->new
+       (namespace => 'Foo', driver => 'File', root_dir => '/my/root');
+
+=item namespace
+
+A map of namespace names to parameter hashrefs. When you create a cache object
+with the specified namespace, the hashref of parameters will be applied as
+defaults. e.g. if you define
+
+    namespace => {
+        'Foo' => { driver => 'File', root_dir => '/my/root' },
+        'Bar' => { storage => 'database' },
+        ...
+    }
+
+then
+
+    my $cache1 = My::CHI->new
+       (namespace => 'Foo');
+    my $cache2 = My::CHI->new
+       (namespace => 'Bar');
+
+is equivalent to
+
+    my $cache1 = My::CHI->new
+       (namespace => 'Foo', driver => 'File', root_dir => '/my/root');
+    my $cache2 = My::CHI->new
+       (namespace => 'Bar', storage => 'database');
+
+=item defaults
+
+A hash of parameters that will be used as core defaults for all cache objects
+created under this root class. e.g.
+
+    defaults => {
+        on_get_error => 'die',
+        expires_variance => 0.2,
+    }
+
+These can be overridden by namespace defaults, storage settings, or C<new>
+parameters.
+
+=item memoize_cache_objects
+
+True or false, indicates whether C<My::CHI-E<gt>new> should memoize and return
+the same cache object if given the same parameters. This can speed things up if
+you create cache objects frequently. Will currently only work for 0- or 1- key
+parameter hashes. e.g.
+
+    My::CHI->config({
+        memoize_cache_objects => 1,
+    });
+
+then
+
+    # $cache1 and $cache2 will be the same object, regardless of what
+    # namespace and storage defaults are associated with 'Foo'
+    #
+    my $cache1 = My::CHI->new(namespace => 'Foo');
+    my $cache2 = My::CHI->new(namespace => 'Foo');
+
+    # $cache3 and $cache4 will be different objects
+    #
+    my $cache3 = My::CHI->new
+       (namespace => 'Bar', driver => 'File', root_dir => '/my/root');
+    my $cache4 = My::CHI->new
+       (namespace => 'Bar', driver => 'File', root_dir => '/my/root');
+
+To clear the memoized cache objects, call
+
+    My::CHI->clear_memoized_cache_objects;
+
+=back
+
+=head2 How defaults are combined
+
+Defaults are applied in the following order, from highest to lowest precedence:
+
+=over
+
+=item *
+
+Parameters passed in C<new>
+
+=item *
+
+Namespace defaults, if any
+
+=item *
+
+Storage settings, if any
+
+=item *
+
+Core defaults defined under 'defaults'
+
+=back
+
+=head2 Inheritance of config
+
+A subclass will automatically inherit the configuration of its parent if it
+does not call C<config> itself (ala
+L<Class::Data::Inheritable|Class::Data::Inheritable>).
+
+=head2 Reading config from a file
+
+    use YAML::XS qw(LoadFile);
+
+    __PACKAGE__->config(LoadFile("/path/to/cache.yml"));
+
 =for readme continue
 
 =head1 AVAILABILITY OF DRIVERS
@@ -930,6 +1327,11 @@ L<CHI::Driver::Memory|CHI::Driver::Memory> - In-process memory based cache
 
 =item *
 
+L<CHI::Driver::RawMemory|CHI::Driver::RawMemory> - In-process memory based
+cache that stores references directly instead of serializing/deep-copying
+
+=item *
+
 L<CHI::Driver::File|CHI::Driver::File> - File-based cache using one file per
 entry in a multi-level directory structure
 
@@ -968,6 +1370,14 @@ L<CHI::Driver::DBI|CHI::Driver::DBI> - Cache in any DBI-supported database
 
 L<CHI::Driver::BerkeleyDB|CHI::Driver::BerkeleyDB> - Cache in BerkeleyDB files
 
+=item *
+
+L<CHI::Driver::Redis|CHI::Driver::Redis> - Cache in L<Redis|http://redis.io/>
+
+=item *
+
+L<CHI::Driver::SharedMem|CHI::Driver::SharedMem> - Cache in shared memory
+
 =back
 
 This list is likely incomplete. A complete set of drivers can be found on CPAN
@@ -975,6 +1385,14 @@ by searching for "CHI::Driver".
 
 =for readme stop
 
+=head1 PERFORMANCE COMPARISON OF DRIVERS
+
+See L<CHI::Benchmarks> for a comparison of read/write times of both CHI and
+non-CHI cache implementations.
+
+C<etc/bench/bench.pl> in the C<CHI> distribution contains a script to run these
+types of benchmarks on your own system.
+
 =head1 DEVELOPING NEW DRIVERS
 
 See L<CHI::Driver::Development|CHI::Driver::Development> for information on
@@ -983,7 +1401,7 @@ developing new drivers.
 =head1 LOGGING
 
 C<CHI> uses L<Log::Any|Log::Any> for logging events. For example, a debug log
-message is sent for every cache get and set.
+message is sent with category C<CHI::Driver> for every cache get and set.
 
 See L<Log::Any|Log::Any> documentation for how to control where logs get sent,
 if anywhere.
@@ -1018,9 +1436,6 @@ the built-in driver classes. Method calls are kept to a minimum, data is only
 serialized when necessary, and metadata such as expiration time is stored in
 packed binary format alongside the data.
 
-As an example, using Rob Mueller's cacheperl benchmarks, CHI's file driver runs
-3 to 4 times faster than Cache::FileCache.
-
 =item Ease of subclassing
 
 New Cache::Cache subclasses can be tedious to create, due to a lack of code
@@ -1067,7 +1482,8 @@ handle expiration times. Here's how CHI resolves these overlaps.
 =item Serialization
 
 CHI handles its own serialization, passing a flat binary string to the
-underlying cache backend.
+underlying cache backend. The notable exception is
+L<CHI::Driver::RawMemory|CHI::Driver::RawMemory> which does no serialization.
 
 =item Expiration
 
@@ -1100,43 +1516,13 @@ mailing list:
 Bugs and feature requests will be tracked at RT:
 
     http://rt.cpan.org/NoAuth/Bugs.html?Dist=CHI
+    bug-chi@rt.cpan.org
 
 The latest source code can be browsed and fetched at:
 
     http://github.com/jonswar/perl-chi/tree/master
     git clone git://github.com/jonswar/perl-chi.git
 
-=head1 TODO
-
-=over
-
-=item *
-
-Perform cache benchmarks comparing both CHI and non-CHI cache implementations
-
-=item *
-
-Release BerkeleyDB drivers as separate CPAN distributions
-
-=item *
-
-Add docs comparing various strategies for reducing miss stampedes and cost of
-recomputes
-
-=item *
-
-Add expires_next syntax (e.g. expires_next => 'hour')
-
-=item *
-
-Support automatic serialization and escaping of keys
-
-=item *
-
-Create XS versions of main functions in Driver.pm (e.g. get, set)
-
-=back
-
 =head1 ACKNOWLEDGMENTS
 
 Thanks to Dewitt Clinton for the original Cache::Cache, to Rob Mueller for the
@@ -1147,24 +1533,19 @@ CHI was originally designed and developed for the Digital Media group of the
 Hearst Corporation, a diversified media company based in New York City.  Many
 thanks to Hearst management for agreeing to this open source release.
 
-=head1 AUTHOR
-
-Jonathan Swartz
-
 =head1 SEE ALSO
 
-L<Cache::Cache|Cache::Cache>, L<Cache::Memcached|Cache::Memcached>,
-L<Cache::FastMmap|Cache::FastMmap>
+L<Cache::Cache|Cache::Cache>
 
-=head1 COPYRIGHT & LICENSE
+=head1 AUTHOR
 
-Copyright (C) 2007 Jonathan Swartz.
+Jonathan Swartz <swartz@pobox.com>
 
-CHI is provided "as is" and without any express or implied warranties,
-including, without limitation, the implied warranties of merchantibility and
-fitness for a particular purpose.
+=head1 COPYRIGHT AND LICENSE
 
-This program is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself.
+This software is copyright (c) 2021 by Jonathan Swartz.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
