@@ -1,5 +1,5 @@
 package CHI::t::Driver;
-$CHI::t::Driver::VERSION = '0.60';
+$CHI::t::Driver::VERSION = '0.61';
 use strict;
 use warnings;
 use CHI::Test;
@@ -584,7 +584,7 @@ sub test_serialize : Tests {
 
 {
     package DummySerializer;
-$DummySerializer::VERSION = '0.60';
+$DummySerializer::VERSION = '0.61';
 sub serialize   { }
     sub deserialize { }
 }
@@ -1277,13 +1277,13 @@ sub test_stats : Tests {
     $log->empty_ok();
     $stats->flush();
     $log->contains_ok(
-        qr/CHI stats: {"absent_misses":2,"end_time":\d+,"expired_misses":1,"get_time_ms":\d+,"hits":1,"label":"$label","namespace":"Foo","root_class":"CHI","set_key_size":6,"set_time_ms":\d+,"set_value_size":20,"sets":1,"start_time":\d+}/
+        qr/CHI stats: \{"absent_misses":2,"end_time":\d+,"expired_misses":1,"get_time_ms":\d+,"hits":1,"label":"$label","namespace":"Foo","root_class":"CHI","set_key_size":6,"set_time_ms":\d+,"set_value_size":20,"sets":1,"start_time":\d+\}/
     );
     $log->contains_ok(
-        qr/CHI stats: {"end_time":\d+,"label":"$label","namespace":"Bar","root_class":"CHI","set_key_size":12,"set_time_ms":\d+,"set_value_size":52,"sets":2,"start_time":\d+}/
+        qr/CHI stats: \{"end_time":\d+,"label":"$label","namespace":"Bar","root_class":"CHI","set_key_size":12,"set_time_ms":\d+,"set_value_size":52,"sets":2,"start_time":\d+\}/
     );
     $log->contains_ok(
-        qr/CHI stats: {"absent_misses":1,"compute_time_ms":\d+,"computes":1,"end_time":\d+,"get_time_ms":\d+,"hits":2,"label":"$label","namespace":"Baz","root_class":"CHI","set_key_size":6,"set_time_ms":\d+,"set_value_size":44,"sets":1,"start_time":\d+}/
+        qr/CHI stats: \{"absent_misses":1,"compute_time_ms":\d+,"computes":1,"end_time":\d+,"get_time_ms":\d+,"hits":2,"label":"$label","namespace":"Baz","root_class":"CHI","set_key_size":6,"set_time_ms":\d+,"set_value_size":44,"sets":1,"start_time":\d+\}/
     );
     $log->empty_ok();
 
@@ -1708,7 +1708,7 @@ sub test_no_leak : Tests {
 
 {
     package My::CHI;
-$My::CHI::VERSION = '0.60';
+$My::CHI::VERSION = '0.61';
 our @ISA = qw(CHI);
 }
 
