@@ -1,4 +1,5 @@
 package CHI::t::Driver::File::DepthZero;
+$CHI::t::Driver::File::DepthZero::VERSION = '0.61';
 use strict;
 use warnings;
 use CHI::Test;
@@ -16,7 +17,7 @@ sub new_cache_options {
     return ( $self->SUPER::new_cache_options(), depth => 0 );
 }
 
-sub test_default_depth : Test(2) {
+sub test_default_depth : Tests {
     my $self = shift;
 
     my $cache = $self->new_cache();
